#!/bin/env python

# File : lostparadise.py
# Version : 0.3

# (c) 2000, Free Software Foundation
#
# lostparadise is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# lostparadise is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with lostparadise; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 
# Auteur : Johann Dro <nojhan@ifrance.com>

import string, urllib, cmd, os, sys, re


class LostParadise:
    """Permet de rcuprer la chanson actuellement joue sur radioparadise et de la sauvegarder"""
    def __init__(self):

        # expressions rationnelles correspondantes aux infos
        # page de la chanson
        self.song_page_re = re.compile("\d*:\d* .m - <a href=\""
                                       +"(?P<songurl>modules.php\?name=songinfo&song_id=\d*)"
                                       +"\">$")
        # groupe et titre de la chanson
        self.song_re = re.compile("(?P<songband>.*)\s*-\s*(?P<songtitle>.*)</a><br>$")
        # titre et url de l'image de l'album
        self.album_re = re.compile('<table border=\"0\" width=\"100%\" cellpadding=\"0\" cellspacing=\"0\"><tr><td class=\"small\"><a href=\"modules.php\?name=Amazon&amp;asin=.*\"><span style=\"font-size:10pt\"><b>(?P<albumtitle>.*)</b></span></a><hr><a href=\"modules.php\?name=Amazon&amp;asin=.*\"><img vspace=\"4\" hspace=\"4\" src=\"(?P<albumimage>http://images.amazon.com/images/.*)\" border =\"1\" align=\"left\" alt=\".*\" ></a>.*')

        # taille de l'image tlcharge
        # big, normal, small
        self.album_image_size = 'big'
        
        # fichier o enregistrer l'image
        self.album_image_file = 'lostparadise_current_image.jpg'
        # navigateur web
        self.browser = 'konqueror'
        # lecteur de musique
        self.player = 'xmms'
        # url de base de RP
        self.url = 'http://www.radioparadise.com/'
        # fichier rss de sauvegarde du best-of
        self.savefile = 'lostparadise.xml'
        # url du flux
        self.listen_url = 'http://www.radioparadise.com/musiclinks/rp_128.m3u'
        # url de la page playlist
        self.playlist_url = self.url + 'modules.php?name=Playlist'

        self.song_title = ""
        self.song_band = ""
        self.song_page = ""
        self.album_title = ""
        self.album_image = ""
        
    def grab(self):
        """Rcupre la playlist et extrait le titre et le lien vers la page de la chanson en cours"""
        # tlcharge la page
        pl_pages = urllib.urlopen(self.playlist_url).readlines()

        # recherche les motifs pour la chanson
        for line in pl_pages:
            findPage = self.song_page_re.search(line)
            findSong = self.song_re.search(line)
            
            if findPage:
                # url de la page de la chanson
                self.song_page = self.url + findPage.group('songurl').strip()

            if findSong:
                # titre de la chanson
                self.song_title = findSong.group('songtitle').strip()
                # groupe de la chanson
                self.song_band = findSong.group('songband').strip()

                # sortie car on ne veut que la premire occurence
                break
                

    def album(self):
        """ Rcupre l'url de l'image de l'album en cours  """
        # tlcharge la page
        pl_pages = urllib.urlopen(self.song_page).readlines()

        # recherche les motifs pour l'album
        for line in pl_pages:
            
            findAlbum = self.album_re.search(line)
            
            if findAlbum:
                # titre de l'album
                self.album_title = findAlbum.group('albumtitle').strip()
                # url de l'image
                self.album_image = findAlbum.group('albumimage')

                # si on demande la grande image
                if self.album_image_size == 'big':
                    self.album_image = self.album_image.replace('THUMB','LZZZZ').strip()
                    
                    # l'image moyenne
                elif self.album_image_size == 'normal':
                    self.album_image = self.album_image.replace('THUMB','MZZZZ').strip()
                    
                    # la petite image
                else:
                    pass
                
                # sortie, on ne veut que la premire occurence
                break
        

    def grabAlbumImage(self):
        """Tlcharge la couverture de l'album"""
        urllib.urlretrieve(self.album_image,self.album_image_file)
        

    def see(self):
        """Affiche simplement le titre de la chanson sur la sortie standard"""
        self.grab()
        print self.song_band,' - ',self.song_title

    def save(self):
        """Enregistre la chanson dans le fichier de sauvegarde"""
        self.see()
            
        rss_file = open(self.savefile,'r')
        rss = rss_file.readlines()
        rss_file.close()
        
        rss_item = ['<item>\n','<title>'+self.song_band+' - '+self.song_title+'</title>\n','<link>'+self.song_page+'</link>\n','</item>\n']
        
        rss = rss[:len(rss)-2]+rss_item+rss[len(rss)-2:len(rss)]
        
        rss_file=open(self.savefile,'w')
        rss_file.writelines(rss)

class LPShell(cmd.Cmd):
    """Interface en ligne de commande toute simple"""

    def help_help(self):
        print "Affiche la liste des commandes disposants d'une aide"

    def do_ls(self,line):
        """Affiche la chanson en cours"""
        print 'Patientez...',
        lp = LostParadise()
        print '\r',
        lp.see()

    def do_cp(self,line):
        """Affiche la chanson en cours et l'enregistre dans le fichier de sauvegarde"""
        print 'Patientez...',
        lp = LostParadise()
        print '\r',
        lp.save()
        print '(Enregistr)'

    def do_cd(self,line):
        """Lance le navigateur sur la page de la chanson en cours"""
        print 'Patientez...',
        lp = LostParadise()
        print '\r',
        lp.see()
        print '(Lancement de '+lp.browser+')'
        os.system( lp.browser + ' "' + lp.song_page + '" > /dev/null' )
        
    def do_EOF(self, ligne):
        """Commande de sortie du shell : [Ctrl-D] sous Unix, [Ctrl-Z] sous Windows"""
        print '\n'
        sys.exit()
        
    def do_mv(self,line):
        """Lance le lecteur multimdia sur la radio"""
        print 'Patientez...',
        lp = LostParadise()
        print '\r',
        lp.see()
        print '(Lancement de '+lp.browser+')'
        os.system( lp.browser + ' "' + lp.listen_url + '" > /dev/null' )
        print 'faites [Ctrl-C]'
        
        

# en cas d'appel en ligne de commande
if __name__=='__main__':

    if len(sys.argv)>1:
        if sys.argv[1]=='-i' or sys.argv[1]=='--interactive':
            print 'Bienvenu sur le shell de LostParadise'
            print '    LostParadise est un petit outil pour mmoriser des titres passant sur RadioParadise'
            print '    tappez "help" pour avoir la liste des commandes'
            print '    web site de lostparadise : http://nojhan.free.fr/article.php?id_article=36'
            
            shell = LPShell()
            shell.prompt = "LostParadise> "
            shell.cmdloop()
    else:
        lp = LostParadise()
        lp.grab()
        lp.album()
        lp.grabAlbumImage()
        print '"%s" "%s" "%s" %s' % (lp.song_band,lp.song_title,lp.album_title,lp.album_image_file)


# exemple de fichier RSS
"""
<rss version="2.0">
 <channel>
  <title>RadioParadise</title>
  <link>http://www.radioparadise.com/</link>
 </channel>
 <item>
  <title>Bruce Cockburn - Mango</title>
  <link>http://www.radioparadise.com/modules.php?name=songinfo&song_id=1129</link>
 </item>
 <item>
  <title>Traffic - Freedom Rider</title>
  <link>http://www.radioparadise.com/modules.php?name=songinfo&song_id=585</link>
 </item>
</rss>
"""
